BASE_URL = 'http://www.pro-football-reference.com'

import decorators
import utils
import boxscores
import players
import teams
import finders

# modules/variables to expose
__all__ = [
    'BASE_URL',
    'finders',
    'decorators',
    'utils',
    'boxscores'
    'players',
    'teams',
]
